import 'dart:developer';

import 'package:flutter/material.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:google_fonts/google_fonts.dart';
import 'package:hub/app_state.dart';
import 'package:hub/backend/api_requests/api_calls.dart';
import 'package:hub/backend/api_requests/api_manager.dart';
import 'package:hub/components/molecular_components/message_opt_modal/opt_modal_widget.dart';
import 'package:hub/components/molecular_components/option_selection_modal/option_selection_modal_widget.dart';
import 'package:hub/components/templates_components/card_item_template_component/card_item_template_component_widget.dart';
import 'package:hub/flutter_flow/custom_functions.dart';
import 'package:hub/flutter_flow/flutter_flow_icon_button.dart';
import 'package:hub/flutter_flow/flutter_flow_model.dart';

import 'package:hub/flutter_flow/flutter_flow_theme.dart';
import 'package:hub/flutter_flow/flutter_flow_util.dart';
import 'package:hub/flutter_flow/internationalization.dart';
import 'package:hub/pages/acess_history_page/acess_history_page_model.dart';
import 'package:rxdart/rxdart.dart';

@immutable
class AcessHistoryPageWidget extends StatefulWidget {
  late Map<String, String> opt = {
    'personType': '.*',
    'accessType': '.*',
    'search': '.*',
  };
  AcessHistoryPageWidget({super.key, required this.opt});
  @override
  State<AcessHistoryPageWidget> createState() =>
      _AcessHistoryPageWidgetState(opt);
}

class AccessHistoryItemWidget extends StatelessWidget {
  final dynamic accessHistoryItem;

  const AccessHistoryItemWidget({Key? key, required this.accessHistoryItem})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.all(8.0),
      child: Column(),
    );
  }
}

class _AcessHistoryPageWidgetState extends State<AcessHistoryPageWidget> {
  late AcessHistoryPageModel _model;
  final BehaviorSubject<Map<String, String>> selectedTypeSubject;
  bool _isSubjectClosed = false;
  final scaffoldKey = GlobalKey<ScaffoldState>();

  _AcessHistoryPageWidgetState(Map<String, String> opt)
      : selectedTypeSubject = BehaviorSubject.seeded(opt) {
    selectedTypeSubject.listen((value) {});
  }

  @override
  void initState() {
    super.initState();
    _model = createModel(context, () => AcessHistoryPageModel());
  }

  @override
  void dispose() {
    selectedTypeSubject.close();
    _isSubjectClosed = true;
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        key: scaffoldKey,
        backgroundColor: FlutterFlowTheme.of(context).primaryBackground,
        appBar: _appBarOrganismWidget(context),
        body: _accessHistoryListOrganismWidget(context));
  }

// AppBar Widgets
  PreferredSizeWidget _appBarOrganismWidget(BuildContext context) {
    final theme = FlutterFlowTheme.of(context);
    return AppBar(
      backgroundColor: theme.primaryBackground,
      automaticallyImplyLeading: false,
      leading: _appBarBackButtonAtomWidget(context, theme),
      title: _appBarTitleMoleculeWidget(context, theme),
      centerTitle: true,
      elevation: 0.0,
      actions: [
        _appBarFilterButtonAtomWidget(context),
      ],
    );
  }

  Widget _appBarBackButtonAtomWidget(
      BuildContext context, FlutterFlowTheme theme) {
    return FlutterFlowIconButton(
      borderColor: Colors.transparent,
      borderRadius: 30.0,
      borderWidth: 1.0,
      buttonSize: 60.0,
      icon: Icon(
        Icons.keyboard_arrow_left,
        color: theme.primaryText,
        size: 30.0,
      ),
      onPressed: () => Navigator.of(context).pop(),
    );
  }

  Widget _appBarTitleMoleculeWidget(
      BuildContext context, FlutterFlowTheme theme) {
    return Text(
      FFLocalizations.of(context).getText('ch8qymga'),
      style: theme.headlineMedium.override(
        fontFamily: theme.headlineMediumFamily,
        color: theme.primaryText,
        fontSize: 16.0,
        letterSpacing: 0.0,
        useGoogleFonts:
            GoogleFonts.asMap().containsKey(theme.headlineMediumFamily),
      ),
    );
  }

  Widget _appBarFilterButtonAtomWidget(BuildContext context) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.end,
      children: [
        IconButton(
          icon: const Icon(Icons.filter_list),
          padding: EdgeInsets.fromLTRB(0, 0, 10, 0),
          onPressed: () async {
            final Map<String, String>? selectedFilter =
                await showModalBottomSheet<Map<String, String>>(
              isScrollControlled: true,
              backgroundColor: Colors.transparent,
              context: context,
              builder: (context) => OptModalWidget(
                defaultAccessType:
                    selectedTypeSubject.value['accessType'] ?? '.*',
                defaultPersonType:
                    selectedTypeSubject.value['personType'] ?? '.*',
              ),
            );

            if (selectedFilter != null) {
              _updateAccessHistoryAction(selectedFilter);
            }
          },
        ),
      ],
    );
  }

  void _updateAccessHistoryAction(Map<String, String> newType) {
    if (!_isSubjectClosed) {
      final currentType = selectedTypeSubject.value;
      final updatedType = Map<String, String>.from(currentType);
      bool needsUpdate = false;
      newType.forEach((key, newValue) {
        if (currentType[key] != newValue) {
          updatedType[key] = newValue;
          needsUpdate = true;
        }
      });
      if (needsUpdate) {
        selectedTypeSubject.add(updatedType);
        safeSetState(() {});
      }
    }
  }

  Stream<ApiCallResponse> fetchAccessHistoryService(String selectedType) {
    switch (selectedType) {
      case 'E':
        return _model.accessHistory(
          requestFn: () => PhpGroup.getAccessCall.call(
            devUUID: FFAppState().devUUID,
            userUUID: FFAppState().userUUID,
            cliID: FFAppState().cliUUID,
            atividade: 'getAcessos',
            pageSize: '100',
            pageNumber: '1',
            pesTipo: 'E',
          ),
        );
      case 'O':
        return _model.accessHistory(
          requestFn: () => PhpGroup.getAccessCall.call(
            devUUID: FFAppState().devUUID,
            userUUID: FFAppState().userUUID,
            cliID: FFAppState().cliUUID,
            atividade: 'getAcessos',
            pageSize: '100',
            pageNumber: '1',
            pesTipo: 'O',
          ),
        );
      default:
        return _model.accessHistory(
          requestFn: () => PhpGroup.getAccessCall.call(
            devUUID: FFAppState().devUUID,
            userUUID: FFAppState().userUUID,
            cliID: FFAppState().cliUUID,
            atividade: 'getAcessos',
            pageSize: '100',
            pageNumber: '1',
            pesTipo: 'T',
          ),
        );
    }
  }

  Widget _accessHistoryListOrganismWidget(BuildContext context) {
    return SingleChildScrollView(
      child: Column(
        children: [
          StreamBuilder<Map<String, String>>(
            stream: selectedTypeSubject.stream,
            builder: (context, snapshot) {
              if (!snapshot.hasData) {
                return Center(child: CircularProgressIndicator());
              }
              final selected = snapshot.data!;
              return _cardListViewOrganismWidget(selected);
            },
          ),
        ],
      ),
    );
  }

  Future<List<dynamic>> fetchCardListViewService(
      Map<String, String> select) async {
    final response =
        await fetchAccessHistoryService(select['personType']!).first;
    final List<dynamic> accessHistory = response.jsonBody['acessos'] ?? [];

    return accessHistory.where((item) {
      final personTypeMatches = select['personType'] == '.*' ||
          item["PES_TIPO"].toString() == select['personType'];
      final accessTypeMatches = select['accessType'] == '.*' ||
          item["ACE_TIPO"].toString() == select['accessType'];
      final searchMatches = select['search'] == '.*' ||
          item["PES_NOME"]
              .toString()
              .toLowerCase()
              .contains(select['search']!.toLowerCase());
      return personTypeMatches && accessTypeMatches && searchMatches;
    }).toList();
  }

  Widget _cardListViewOrganismWidget(Map<String, String> selected) {
    return FutureBuilder<List<dynamic>>(
      future: fetchCardListViewService(selected),
      builder: (context, snapshot) {
        if (snapshot.connectionState == ConnectionState.waiting) {
          return Center(
            child: SizedBox(
              width: 50.0,
              height: 50.0,
              child: SpinKitCircle(
                color: FlutterFlowTheme.of(context).primary,
                size: 50.0,
              ),
            ),
          );
        } else if (snapshot.hasError) {
          return Text('Error: ${snapshot.error}');
        } else {
          final accessHistory = snapshot.data!;
          return ListView.builder(
            shrinkWrap: true,
            physics: const NeverScrollableScrollPhysics(),
            itemCount: accessHistory.length,
            itemBuilder: (context, index) {
              final accessHistoryItem = accessHistory[index];
              return _accessHistoryCardMoleculeWidget(
                  context, accessHistoryItem);
            },
          );
        }
      },
    );
  }

  Widget _accessHistoryCardMoleculeWidget(
      BuildContext context, dynamic accessHistoryItem) {
    return CardItemTemplateComponentWidget(
        imageHashMap: Map<String, String>.from({
          'key': accessHistoryItem['PES_ID'] ?? '',
          'value': accessHistoryItem['PES_TIPO'] ?? '',
        }),
        labelsHashMap: Map<String, String>.from({
          'Nome:': accessHistoryItem['PES_NOME'] ?? '',
          'Acesso:': accessHistoryItem['ACE_DATAHORA'] ?? '',
          'Setor:': accessHistoryItem['SET_DESCRICAO'] ?? '',
        }),
        statusHashMap: [
          accessHistoryItem['PES_TIPO'] == 'O'
              ? Map<String, Color>.from({
                  FFLocalizations.of(context).getVariableText(
                    ptText: 'Morador',
                    enText: 'Resident',
                  ): FlutterFlowTheme.of(context).alternate2,
                })
              : accessHistoryItem['PES_TIPO'] == 'E'
                  ? Map<String, Color>.from({
                      FFLocalizations.of(context).getVariableText(
                        ptText: 'Visitante',
                        enText: 'Visitor',
                      ): FlutterFlowTheme.of(context).alternate2,
                    })
                  : Map<String, Color>.from({
                      FFLocalizations.of(context).getVariableText(
                        ptText: 'Desconhecido',
                        enText: 'Unknown',
                      ): FlutterFlowTheme.of(context).alternate2,
                    }),
          accessHistoryItem['ACE_TIPO'] == '0'
              ? Map<String, Color>.from({
                  FFLocalizations.of(context).getVariableText(
                    ptText: 'Entrada',
                    enText: 'Entrance',
                  ): FlutterFlowTheme.of(context).success,
                })
              : accessHistoryItem['ACE_TIPO'] == '1'
                  ? Map<String, Color>.from({
                      FFLocalizations.of(context).getVariableText(
                        ptText: 'Saída',
                        enText: 'Exit',
                      ): FlutterFlowTheme.of(context).error,
                    })
                  : Map<String, Color>.from({
                      FFLocalizations.of(context).getVariableText(
                        ptText: 'Desconhecido',
                        enText: 'Unknown',
                      ): FlutterFlowTheme.of(context).warning,
                    })
        ],
        onTapCardItemAction: () async {});
  }
}

String imageUrlAtomWidget(String document, String type) {
  return valueOrDefault<String>(
    "https://freaccess.com.br/freaccess/getImage.php?&cliID=${FFAppState().cliUUID}&atividade=getFoto&Documento=$document&tipo=$type",
    "https://storage.googleapis.com/flutterflow-io-6f20.appspot.com/projects/flutter-freaccess-hub-0xgz9q/assets/7ftdetkzc3s0/360_F_64676383_LdbmhiNM6Ypzb3FM4PPuFP9rHe7ri8Ju.jpg",
  );
}
